/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.commons.compress.archivers.ar;

import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.ArchiveInputStream;

public class ArArchiveInputStream extends ArchiveInputStream {

	private final InputStream input;
	private long offset = 0;
	
	public ArArchiveInputStream( final InputStream pInput ) {
		input = pInput;
	}
	
	public ArchiveEntry getNextEntry() throws IOException {
		
		if (offset == 0) {
			final byte[] expected = "!<arch>\n".getBytes();			
			final byte[] realized = new byte[expected.length]; 
			final int read = input.read(realized);
			if (read != expected.length) {
				throw new IOException("failed to read header");
			}
			for (int i = 0; i < expected.length; i++) {
				if (expected[i] != realized[i]) {
					throw new IOException("invalid header " + new String(realized));
				}
			}
		}

		if (input.available() == 0) {
			return null;
		}
				
		if (offset % 2 != 0) {
			read();
		}

		final byte[] name = new byte[16];
		final byte[] lastmodified = new byte[12];
		final byte[] userid = new byte[6];
		final byte[] groupid = new byte[6];
		final byte[] filemode = new byte[8];
		final byte[] length = new byte[10];
		
		read(name);
		read(lastmodified);
		read(userid);
		read(groupid);
		read(filemode);
		read(length);

		{
			final byte[] expected = "`\012".getBytes();			
			final byte[] realized = new byte[expected.length]; 
			final int read = input.read(realized);
			if (read != expected.length) {
				throw new IOException("failed to read entry header");
			}
			for (int i = 0; i < expected.length; i++) {
				if (expected[i] != realized[i]) {
					throw new IOException("invalid entry header. not read the content?");
				}
			}
		}
		
		return new ArArchiveEntry(new String(name).trim(), Long.parseLong(new String(length).trim()));
	
	}
	
	
	public int read() throws IOException {
		final int ret = input.read();
		offset++;
		return ret;
	}

	public int read(byte[] b, int off, int len) throws IOException {
		return this.input.read(b, off, len);
	}
	
	public static boolean matches( byte[] signature ) {
		// 3c21 7261 6863 0a3e
    	
    	if (signature[0] != 0x21) {
    		return false;
    	}
    	if (signature[1] != 0x3c) {
    		return false;
    	}
    	if (signature[2] != 0x61) {
    		return false;
    	}
    	if (signature[3] != 0x72) {
    		return false;
    	}
    	if (signature[4] != 0x63) {
    		return false;
    	}
    	if (signature[5] != 0x68) {
    		return false;
    	}
    	if (signature[6] != 0x3e) {
    		return false;
    	}
    	if (signature[7] != 0x0a) {
    		return false;
    	}
    	
    	return true;
	}

}
